import pytest

from pathlib import Path

import pinetext.client as client_mod
from pinetext.client import PineText


@pytest.fixture
def cli(monkeypatch):
    def dummy_run(self):
        return

    monkeypatch.setattr(
        "pinetext.client.PineText.run",
        dummy_run,
    )


@pytest.fixture
def pinetext(monkeypatch):
    class DummyAssistant:
        def __init__(self):
            self.files = []

        def create_assistant(self, assistant_name: str, instructions: str = None):
            return self

        def describe_assistant(self, assistant_name: str):
            return self

        def list_files(self):
            return self.files

        def upload_file(self, file_path: str, metadata=None, timeout=None):
            self.files.append(Path(file_path).name)

        def chat(self, messages, model=None):
            class Message:
                def __init__(self, content):
                    self.content = content

            class Response:
                def __init__(self, content):
                    self.message = Message(content)

            return Response("Test")

    assistant = DummyAssistant()

    class DummyPinecone:
        def __init__(self, api_key):
            self.assistant = assistant

    monkeypatch.setattr(client_mod, "Pinecone", DummyPinecone)

    client = PineText()
    client.assistant = assistant
    client.pinecone = DummyPinecone(None)
    return client



from pydantic import BaseModel
from pydantic_settings import BaseSettings, SettingsConfigDict


class Pinecone(BaseModel):
    api_key: str | None = None
    assistant: str | None = "test-assistant"
    data_dir: str | None = "data"
    model: str | None = "o4-mini"


class WandB(BaseModel):
    api_key: str | None = None
    project: str | None = "pinetext"


class Settings(BaseSettings):
    pinecone: Pinecone = Pinecone()
    wandb: WandB = WandB()
    model_config = SettingsConfigDict(
        env_file=".env",
        env_file_encoding="utf-8",
        env_nested_delimiter="__",
        env_prefix="PINETEXT_",
    )



import builtins


def test_get_or_create_assistant(pinetext):
    assistant = pinetext.get_or_create_assistant("foo")
    assert assistant is pinetext.pinecone.assistant


def test_upload_files(pinetext, tmp_path):
    data = tmp_path / "data"
    data.mkdir()
    (data / "test.txt").write_text("TEST")
    pinetext.upload_files(str(data))
    assert "test.txt" in pinetext.pinecone.assistant.list_files()


def test_chat(pinetext):
    resp = pinetext.chat("This is a test", model="test-model")
    assert resp == "Test"


def test_run(pinetext, monkeypatch):
    monkeypatch.setattr(builtins, "input", lambda prompt="": next(iter(["exit"])))
    res = pinetext.run()
    assert res is None



import wandb
import weave

from pathlib import Path

from pinecone import Pinecone
from pinecone_plugins.assistant.models.chat import Message

from pinetext.settings import Settings


settings = Settings()


class PineText:
    def __init__(self):
        pass

    def get_or_create_assistant(self, name: str):
        try:
            return self.pinecone.assistant.describe_assistant(assistant_name=name)
        except Exception:
            return self.pinecone.assistant.create_assistant(assistant_name=name)

    def upload_files(self, path: str):
        folder = Path(path)
        if folder.is_dir():
            uploaded = [x.name for x in self.assistant.list_files()]
            for x in sorted(folder.iterdir()):
                if x.name not in uploaded:
                    self.assistant.upload_file(
                        file_path=str(x.resolve()),
                        metadata={
                            "filename": x.name,
                            "extension": x.suffix.lower().lstrip("."),
                        },
                        timeout=None,
                    )

    @weave.op()
    def chat(self, text: str, model: str):
        msg = Message(role="user", content=text)
        resp = self.assistant.chat(messages=[msg], model=model)
        return resp.message.content

    def run(self):
        if settings.wandb.api_key:
            wandb.login(key=settings.wandb.api_key)
            weave.init(settings.wandb.project)
        self.pinecone = Pinecone(api_key=settings.pinecone.api_key)
        self.assistant = self.get_or_create_assistant(settings.pinecone.assistant)
        self.upload_files(settings.pinecone.data_dir)

        while True:
            text = input("> ").strip()
            if text.lower() in ("exit", "quit"):
                break
            res = self.chat(text, settings.pinecone.model)
            print(res)
